// This file is generated by kconfig_compiler_kf6 from keditbookmarks.kcfg.
// All changes you do to this file will be lost.
#ifndef KEBSETTINGS_H
#define KEBSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class KEBSettings : public KConfigSkeleton
{
  public:

    static KEBSettings *self();
    ~KEBSettings() override;

    /**
      Set Name
    */
    static
    void setName( int v )
    {
      if (!self()->isNameImmutable())
        self()->mName = v;
    }

    /**
      Get Name
    */
    static
    int name()
    {
      return self()->mName;
    }

    /**
      Is Name Immutable
    */
    static
    bool isNameImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Name" ) );
    }

    /**
      Set URL
    */
    static
    void setURL( int v )
    {
      if (!self()->isURLImmutable())
        self()->mURL = v;
    }

    /**
      Get URL
    */
    static
    int uRL()
    {
      return self()->mURL;
    }

    /**
      Is URL Immutable
    */
    static
    bool isURLImmutable()
    {
      return self()->isImmutable( QStringLiteral( "URL" ) );
    }

    /**
      Set Comment
    */
    static
    void setComment( int v )
    {
      if (!self()->isCommentImmutable())
        self()->mComment = v;
    }

    /**
      Get Comment
    */
    static
    int comment()
    {
      return self()->mComment;
    }

    /**
      Is Comment Immutable
    */
    static
    bool isCommentImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Comment" ) );
    }

    /**
      Set Status
    */
    static
    void setStatus( int v )
    {
      if (!self()->isStatusImmutable())
        self()->mStatus = v;
    }

    /**
      Get Status
    */
    static
    int status()
    {
      return self()->mStatus;
    }

    /**
      Is Status Immutable
    */
    static
    bool isStatusImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Status" ) );
    }

    /**
      Set Address
    */
    static
    void setAddress( int v )
    {
      if (!self()->isAddressImmutable())
        self()->mAddress = v;
    }

    /**
      Get Address
    */
    static
    int address()
    {
      return self()->mAddress;
    }

    /**
      Is Address Immutable
    */
    static
    bool isAddressImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Address" ) );
    }

    /**
      Set Save On Close
    */
    static
    void setSaveOnClose( bool v )
    {
      if (!self()->isSaveOnCloseImmutable())
        self()->mSaveOnClose = v;
    }

    /**
      Get Save On Close
    */
    static
    bool saveOnClose()
    {
      return self()->mSaveOnClose;
    }

    /**
      Is Save On Close Immutable
    */
    static
    bool isSaveOnCloseImmutable()
    {
      return self()->isImmutable( QStringLiteral( "SaveOnClose" ) );
    }

  protected:
    KEBSettings();
    friend class KEBSettingsHelper;


    // Columns
    int mName;
    int mURL;
    int mComment;
    int mStatus;
    int mAddress;

    // General
    bool mSaveOnClose;
};

#endif

